Release notes
====

0.4.13 (2020-09-04)
---

Updated the rendering system on Linux to allow selecting between different backends.

By default, Storm now utilizes the same background that Gtk+ uses, which is usually hardware
accelerated in some way (for example, on X11, the Cairo manual states that the XRender extension is
used if it is available). This has the benefit of improving the copying from the rendering buffer to
the window, as the buffers usually reside in the same location. It also avoid crashes when running
OpenGL against certain drivers. A software fallback is also available.

It is possible to select which backend to use by setting an environment variable, as described in
the documentation for the GUI library, if OpenGL rendering is still desired.

0.4.12 (2020-09-04)
---

- Fixed a bug that caused a resource leak when spawning UThreads in certain circumstances.

0.4.11 (2020-07-05)
---

- The GUI library is DPI-aware on Windows.

0.4.10 (2020-06-28)
---

- Progvis is now able to track reads and writes to memory to detect data races.
- Improved the C++ front-end used by Progvis.
- Performance improvements in the parser, mainly the creation of parse tables is faster.
- `break` and `continue` may be used to end a `unless` block inside a loop.

0.4.9 (2020-04-27)
---

- All named objects may now contain a reference to the source where that entity is defined.
- All entities defined in C++ contain a reference to where they are declared.
- The help text shows the source location of the entity.
- Nested types declared as private in C++ are private in Storm as well.
- The system attempts to print a stack trace before crashing due to a segmentation fault.

0.4.8 (2020-04-22)
---

- Added default constructor to `Bool` for consistency with other built-in types.
- Improved pointer visualizations in the program visualization tool.
- Allow `NO_STEP` in the C/C++ visualization to control what the visualization shows.

0.4.7 (2020-04-20)
---

- `break` and `continue` works inside for-each loops in Basic Storm.
- `unsafe:RawPtr:asObject` now returns `Maybe<Object>`.
- Improvements to the program visualization.

0.4.6 (2020-04-16)
---

- Fixed a bug in the queue implementation that could cause out of bounds writes.

0.4.5 (2020-04-15)
---

- Minor improvement in the program visualization tool.
- Added `createDir` to the `Url` class.
- Disallow creating "void" variables in Basic Storm. This would previously crash the system. Now an error is generated instead.

0.4.4 (2020-04-05)
---

* Added a `~` separator to the syntax language.
* Improvied the documentation on the syntax language and syntax transformations.
* Robustness improvements to `substr` inside String.
* Made sure that the parent window gains focus after a modal child is closed on Windows.
* Improvements to the program visualization framework.

0.4.3 (2020-03-08)
---

Improvements to the UI library:
- Support for adding menus to windows.
- Support for modal dialog boxes.
- Support scrolling window contents.
- Implemented standard file picker interfaces.
- Implemented a standard "about" dialog in the UI library.
- Minor bugfixes.

Improvements to Basic Storm:
- Allow using absolute names inside the `named{}` syntax. E.g. `named{:foo}`
- Implemented the `continue` and `break` keywords.

Improvements in the core:
- Fixed a bug where accessing the Compiler thread from other libraries could cause crashes.
- Allow static member functions to use the `EnginePtr` interface, just as free functions are able to do.
- Cleanup of local variables in generated code is the same during normal termination as during exception cleanup.

0.4.2 (2020-02-08)
---

- Re-worked how the code generation backend handles cleanup. Now, each variable may be activated at any point in the listing, even if it is not in the topmost block, which greatly simplifies some aspects of code generation on higher levels.
- Basic Storm now encloses each statement in its own block. This means that temporaries created by one statement are destroyed before the next statement is executed rather than at the end of the block.
- Added support for declaring enums in Basic Storm.
- Previously, when using the assignment syntax for variable initialization in Basic Storm, regular constructors could be used to perform the initialization, even if not marked as "cast constructors", which is required for other cases of type conversion. Now, assignment initialization follows the same rules as for regular assignments and type conversions as expected.
- Support chaining `saveTrace` for exceptions, so that it is possible to do `throw SyntaxError(...).saveTrace` to add a stack trace to exceptions that don't normally capture a trace.
- Improved error reporting for the syntax language.

0.4.1 (2020-01-28)
---

- Added `named{}` as a way to refer to the current package using the `lang.bs.macro` extensions to Basic Storm.
- Improved error reporting for the built-in regex engine. Previously, errors were silently ignored which sometimes resulted in surprising behavior.
- Fixed a bug causing `Array.insert` to misbehave when working with arrays of class- or actor types.

0.4.0 (2020-01-27)
---

- Added support for exceptions in the code generation backend. Pointers to Storm objects can be caught regardless of whether they were thrown from C++ or not.
- Added support for exceptions in the Basic Storm frontend.
- Small improvements to the inline-asm language (memory references and `lea` were added).

0.3.0 (2020-01-05)
---

- Added another experimental GC, the Storm Memory Manager, SMM.
- Separated the GC part of the compiler out to a separate library to improve modularity. This also helps in the future when generating stand-alone binaries.

0.2.39 (2019-12-30)
---

- Fixed an issue causing finalizers to not be executed properly.

0.2.38 (2019-12-28)
---

- Implemented a small unit-testing framework for Basic Storm. See the "Libraries" section of the documentation for details.
- Fixed a bug where unused results from calls to other threads would cause an assertion.
- Added an experimental SQL library and language extension by David Ångström, Markus Larsson, Mohammed Hamade, Oscar Falk, Robin Gustavsson, Saima Akhter. See the "Libraries" section of the documentation for details.

0.2.37 (2019-11-07)
---

* It is now possible to use abstract functions in C++ classes exposed to Storm.
* Fixed a potential issue when thread objects were copied before they were properly initialized. This could fix some seemingly random crashes when working with the UI library.

0.2.36 (2019-08-01)
---

- Finalizers are not always executed on new UThreads, which could cause out of memory conditions in some cases. Now finalizer execution is batched, drastically reducing the risk for running out of memory. See commit `14946bf17b5fd74b62fd5b561a133042dbfb861b` for details.
- New debug call: `debug.threadSummary` that shows a stack trace of all running threads in the syste.
- New graph library for the presentation library.

0.2.35 (2019-06-06)
---

* Exposed more of the interface of the newly introduced `Condition` class to Storm, and merged the concept of `Condition` with `WeakCast`.
* `unless` in Basic Storm now supports arbitrary conditions, just like the `if` statement. Previously, only weak casts were supported.

0.2.34 (2019-06-05)
---

* The `iter` member of `WeakSet<T>` now returns a proper iterator instead of void.
* The range-based for loop in Basic Storm supports iterators from `WeakSet<T>` as well.
* Weak casts are allowed in the `while` statement. Previously, only the `if` statement supported weak casts.

0.2.33 (2019-02-06)
---

The GUI library supports Wayland. There may still be some quirks and inefficiencies left.

0.2.32 (2019-01-17)
---

- Added serialization to Storm, explained in detail in the documentation.
- Streamlined the handling of decorators in Basic Storm, and added the ability to define custom decorators for types.
- Fixed a bug that caused memory corruption in rare cases. See commit `926d4251b211a846bb20064fc221979b979454ad` and `517fe86e3ebdf8aca92679998646cda8ead0c402` for details.

0.2.31 (2018-12-23)
---

* Initialization order inside init blocks in Basic Storm now initialize members in the order
  specified inside the init block rather than the order they appear in the class.
* Initialization of inherited classes can be performed in two separate steps, first calling the
  constructor of the super class and then initializing the current class. This allows accessing
  `this` to access the parent class after the constructor of the superclass is called but before the
  object is fully initialized.

0.2.30 (2018-12-09)
---

* `StrBuf` supports formatting floating point numbers.

0.2.29 (2018-12-06)
---

* Fixed an issue where using a locale with a comma as a decimal separator would cause compilation
  errors for float literals on Linux (which could happen after initializing Gtk+).

0.2.28 (2018-12-03)
---

* Added support a limited form of context-dependent grammars to the parser. See the documentation on the syntax language for details.
* Make sure that Storm does not crash when trying to create a global variable with the type `void`.

0.2.27 (2018-11-10)
---

* Added mouse support to the Gui library.
* Improved key handling in the Gui library.
* Fixed string escaping in Basic Storm. Previously, `"` and `$` were not unescaped properly.
* Unary `-` has lower priority than literals in Basic Storm, meaning that `-1` is auto-convertible to a `Float` for example.
* Replaced the old presentation library with a new one.

0.2.26 (2018-10-14)
---

* Calling `randomAccess` on a `IStream` that is not an `RIStream` provides a `LazyMemIStream` which
  buffers the original stream as needed in order to provide random access.

* Fixed an issue where elements returned from a `WeakSet` iterator could refer to finalized
  objects. See commit `0e813305ad89890a53af931482b06e29a5fd46f7` for details.

* Better error messages when a function attempts to override another function with an incompatible
  return type. Previously, this was silently ignored, and the functions were not considered
  compatible. Now, an error message is emitted instead, which makes it easier to spot mistakes.

* Packages now look for a file named `README` that is expected to provide an overview for the entire
  package. The contents of this file are shown in the help for the package, just like comments
  before functions and types.

* A library for generating PDF files was added as `ui.pdf`. The library provides a `Graphics` object
  that draws to a PDF page in the same manner graphics is drawn to the screen.

0.2.25 (2018-10-08)
---

* Expressions with unary operators in Basic Storm now work as expected. Previously `-x + y` would not compile.
* Iterators for the `core.Map` class are now working again. Previously, `begin` and `end` did not return an iterator, but returned `void` instead.
* Functions taking an `EnginePtr` as their first parameter in C++ would sometimes cause the system to crash on X86-64.

0.2.24 (2018-09-29)
---

Bugfixes in the code generation:

* An error in the size of a function's stack frame caused the X64 backend to corrupt variables from time to time.
* Small structures of sizes not natively supported by the CPU (eg. 2 bytes) are now handled properly.

0.2.23 (2018-09-26)
---

* Basic Storm now supports numeric literals in hexadecimal and bitwise operators on unsigned types.* Fixed a bug that caused a crash when the neg op-code was used with a 64-bit operand.
* Proper support for static functions, both inside Storm, in C++ and in Basic Storm.
* More consistent naming of streams in `core.io`.

0.2.22 (2018-08-31)
---

* Improved message when the user types `help` in the Basic Storm repl.
* Minor cleanups of the implementation of `pattern` blocks.
* When using the language server for syntax highlighting, importing non-existing packages with `use` statements
  no longer confuses the highlighting of the rest of the file.
* Made `Frame.size` assignable.

0.2.21 (2018-08-01)
---

* Moved the flag used for marking constructors usable for automatic type casts from `Named` into `Function`.

* Added missing functionality regarding `Double` in the standard library and improved the support
  for doubles in Basic Storm.

* Added the ability to suffix numeric literals with a letter in Basic Storm to indicate the desired
  type of the literal. For example `10i` is always an `Int`, while `10n` is always a `Nat`.

0.2.20 (2018-07-30)
---

* Global variables are initialized lazily by default. Initialization in Basic Storm is still eager,
  but delayed until after all functions are loaded, so that initialization may use functions
  declared in the same package without issues.

* Improved robustness of the pattern syntax. Previously, the pattern syntax assumed that the parse
  tree would be evaluated in the same order as syntax transformations would evaluate all
  nodes. However, for more complex constructs (such as lambda functions) that is not the
  case. Therefore, the pattern syntax now keeps track of which nodes corresponds to which parameters
  to eliminate the issue.

0.2.19 (2018-07-30)
---

* The REPL in Basic Storm now supports creating variables interactively. Variables are either
  declared as regular variables at the top-level, or using assignment. Variables can be listed using
  `variables` and deleted using `del <x>`.

* The REPL in Basic Storm has a new command `help` that lists some important commands
  in the REPL.

* The function pointer syntax in Basic Storm (`&foo(X, Y)`) now evaluates to a function pointer
  taking the exact types specified rather than the types in the declaration of the function. This
  means that the type of such expressions is much easier to reason about. This also allows creating
  function pointers with more restricted types than the original function, which is sometimes
  necessary since the function pointer type is not implicitly convertible in any way, even though
  such conversions would be legal.

* The function pointer syntax in Basic Storm supports automatic deduction of parameter types in some
  cases, just like lambda functions. This makes it possible to omit the parameter list in many
  cases, for example `x.sort(&compare)`.

* Fixed a bug in the allocation of type descriptors in the GC. This bug caused too much memory to be
  initialized, which in some cases corrupted important data structures used by the GC.

0.2.18 (2018-07-19)
---

Added support for global variables.

0.2.17 (2018-07-11)
---

* Added tests for inspecting the raw parse tree from Storm. This was previously used in libraries (such as patterns), but not tested in the test suite.
* Improved the benchmarks in the `benchmark` package.

0.2.16 (2018-06-19)
---

* Basic Storm now resolves parameters and return types to free functions properly. Previously, free functions were unable to use types declared as file private for parameters or return types since the wrong scope was used during resolution of these types.

* The type `lang.bnf.CustomDecl` is now recognized as a possible subclass of `lang.bnf.FileItem`. The `CustomDecl` itself is just a hook that allows arbitrary code to be executed when it is added to a `lang.bnf.FileItem`, but this allows more complex extensions to be written for the syntax language, for example the one found in `test.bnf`.

0.2.15 (2018-06-13)
---

Revised visibility of members in `BSFunction` and `BSCtor`, which caused the layout extension to not compile.

0.2.14 (2018-06-08)
---

Robustness improvements for hash tables containing actors (where hashes depend on the object identity). Previously, some object identities were cleared from the location dependency object even if they were still needed.

0.2.13 (2018-06-08)
---

* The lambda functions in Basic Storm now supports capturing variables from the surrounding scope.
* Memory usage should be reduced slightly, as the syntax trees for compiled functions in Basic Storm are no longer kept around after they have been used.
* Iterators for core.Set<T> were broken. Now they are working properly.

0.2.12 (2018-06-06)
---

* Basic Storm now supports lambda functions. Lambda functions are not yet able to carry any state.
* Improved robustness of parameter handling in inlined functions (see commit `2f27e60a2dcc6a36302850f55b15d247f1982bcc` for details).
* Slight performance improvements, mainly during compiler startup.

0.2.11 (2018-05-07)
---

The type `Maybe<T>` (used for `T?` in Basic Storm) is now extended to support `T` being a value type
in addition to `T` being a class or an actor. For example, it is now possible to use `Maybe<Int>`.

0.2.10 (2018-04-29)
---

Basic Storm supports the keywords `abstract`, `final` and `override` in order to provide additional help to programmers. Corresponding support is also present in Storm itself in the form of flags that can be set on specific functions.

0.2.9 (2018-04-16)
---

Support for TCP sockets is added to the standard library in the package `core.net`. The IO system is also improved to properly handle the asynchronous nature of sockets (eg. closing a socket while a read operation is in progress). Furthermore, it is no longer possible to copy streams based on handles from the operating system, as that would cause race conditions and possibly break the IO system.

0.2.8 (2018-03-31)
---

* It is now possible to create syntax extensions to the syntax language itself.
* Improved the formatting options for the `StrBuf` class. It now supports both
  left and right alignment of output with a specified width.
* Basic Storm supports interpolated strings, such as `"Hello ${name}"`. These expand
  to regular string concatenation using the `#` operator.

0.2.7 (2018-03-25)
---

A library for layout is now included with Storm, including a small domain specific language for
specifying layouts. The library is integrated with the UI library to simplify creation of user
interfaces.

In addition, the following improvements are included:

* Allow nested type definitions in Basic Storm.
* Updated the semantics of function overloading to reduce ambiguity. See the documentation on "function overloading"
  for more details.
* The `help` command in the Storm REPL will load any entity inspected so that all members are listed.

0.2.6 (2018-03-09)
---

It is no longer possible to accidentally share data between threads by reading/writing variables
inside actors. Previously, Basic Storm allowed this without proper synchronization and copying,
which made it possible to accidentally create shared data structures (of non-actor types), which in
turn broke assumptions made by the threading system. From now on, such accesses are synchronized as
if they were done by calling a member function, and thus copies are being made. This does, however,
mean that it is no longer possible to assign directly to these variables. Assignment functions can
be used to circumvent the problem. See the documentation for further details.

On Linux: the GUI library uses a different approach when displaying content rendered using
OpenGL. This approach is more cooperative towards Gtk+, but is slightly more demanding. The problem
with the old approach was that some configurations experienced regular freezes of up to 0.5 s. This
seems to be resolved now.

0.2.5 (2018-03-07)
---

* Added *assignment functions* to Basic Storm (including some support in Storm itself).
* Updated the documentation system to support DocNotes without types (used for displaying which functions are assignment functions).
* Allow clicking on visibility modifiers in the Emacs documentation viewer to go to the documentation for the corresponding type.

0.2.4 (2018-03-03)
---

Storm includes a built-in help system. All objects in the name tree can refer to their documentation.
The documentation is loaded from disk when it is required rather than being kept in memory all the time.
In Basic Storm and C++, documentation is generated from the source code by examining the comment
directly preceding the corresponding declaration, much like Doxygen and Javadoc. However, no special
comment syntax is needed at the moment.

The documentation can be accessed from the Basic Storm REPL using `help core:Str`, or from the Emacs
plugin using `M-x storm-doc`. The Emacs plugin also works properly on Linux now.

0.2.3 (2018-02-04)
---

Improved handling of comparisons throughout Storm and added new functionality to the standard
library.

* The `equals` function in `Object` and `TObject` is removed. Implement `==` for the relevant
  classes instead. This means it is no longer possible to compare two unrelated objects (eg. `Str`
  and `Map<Str>`) for equality since the operation itself does not make sense.
* The `hash` function is no longer present in `Object` and `TObject`, in order to be consistent
  with the behavior `==`.
* `Array<T>.sort()` and `Array<T>.sorted()` were added. These functions are only present when
  `T` contains the function `Bool <(T, T)`, which is used to compare elements. Optionally, a
  `Fn<Bool, T, T>` can be provided to the functions to specify the desired ordering.
* A priority queue `core.PQueue<T>` was added to the standard library. Similarly, the priority
  queue uses `Bool <(T, T)` for comparisons.
* Basic Storm allows operator names for function definitions in order to allow operator overloading.
* Basic Storm automatically falls back on defined operators in case the desired operator is missing.
  Eg. if `a > b` is not present, `b < a` is called instead.
* Basic Storm properly resolves expressions like `1 + 0.3`.

0.2.2 (2018-01-27)
---

All named objects are not always public. It is now possible to associate a `Visibility`
object with each named object to restrict access to them. This is used to implement
the keywords `public`, `protected`, `package` and `private` in Basic Storm.

0.2.1 (2018-01-18)
---

Improved stack traces and stack handling on Windows. Previously, only functions that used exception
handling were shown properly in stack traces (by `core:debug:stackTrace`). This release fixes that
issue by making stack unwinding more similar to the table-based approach used in the Linux version.

The Linux release now includes `libpng` and `libjpeg`, so that a system with incompatible versions
do not cause the `graphics` or the `ui` module to fail to load.

0.2.0 (2018-01-15)
---

The GUI and Sound libraries are now available for Linux as well as Windows. Code using these
libraries should behave mostly the same between the two platforms, even though there are still
differences between the platforms. Some functionality is not yet complete, and some parts could use
some refactoring.

In addition to porting the libraries to Linux, support for more file formats have been added to the
Sound and Graphics libraries. Now, `mp3` and `ppm` files are supported in addition to `ogg`, `flac`,
`wav`, `png`, `jpeg` and `bmp`.

0.1.3 (2017-12-08)
---

Stability improvements in the threading system for all platforms.

- During the initialization of a user level thread, the garbage collector could erroneously try to access the not yet initialized thread which caused a crash.
- During user level thread setup (during what is called *detours* in Storm), the garbage collector could erroneously believe that two user level threads were active at the same time, leading to crashes or unexpected behaviors.

0.1.2 (2017-12-06)
---

Stability improvements for Linux.

- Fixed a threading issue in the IOCondition implementation on Linux. Previously,
  there was a small window right after a thread woke from sleep where the IOCondition
  could enter a state where it ignored 'signal' notifications but always blocked any
  threads calling 'wait'. Since this primitive is used by the threading system in Storm,
  this could cause deadlocks any time threads communicated with each other.

0.1.1 (2017-12-02)
---

- Creating and using a `core:lang:Parser` does not crash anymore. See commit `985481df2253eb1d18506c515875e5908bc95273` for details.
- Shared libraries on Linux does not crash when using the C++ streams (`cout` etc.). See commit `822e7d092ce96ac472aaffdd526d5450c6bb39b4` for details.
- More bullet-proof handling of user thread startups (the mechanism called *detours*). Previously, this could have been an issue when using types with complex copy-constructors. See commit `c06071ba0e322bb9dfb4f985cf6be87a17ceadcf` for details.

0.1.0 (2017-11-14)
---

First release with a version number!

Note that the Linux version of Storm does not yet include the sound and gui libraries available on Windows.

